package jsortdemo.core;

/************************************************************************
  ArrayView.java - Copyright 
 **************************************************************************/


import java.util.*;
import java.awt.*;
import javax.swing.*;

/**
 * Class ArrayView.
 * <p>Project-Title: SortDemo --> ArrayView</p>
 *
 * <p>Description: ArrayView ist die Klasse im Projekt SortDemo, die fuer das
 * Zeichnen der Werte des Arrays und deren jeweiligen Aenderungen im Applet zustaendig ist.
 * Aus diesem Grund erweitert sie JPanel und implementiert Observer</p>
 *
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: </p>
 *
 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */
public class ArrayView extends JPanel implements Observer {

// Fields
private Color BACKGROUND = Color.GRAY;
private Color USUALCOLOR = Color.YELLOW;

private JLabel algoLabel = new JLabel();

protected SortModel model;
protected int viewIndex; 

/* 
 * Verfuegbare Darstellungsarten
 */
static final String[] VIEWS = {
	"Spots",
	"Bars"
};




/**
 * Konstruktor der ArrayView-Klasse
 *
 * @param m Das SortModel, das observiert werden soll
 *
 */
public ArrayView( SortModel m, int viewIndex ) {
	model = m;
	model.addObserver(this);

	add( algoLabel ); 
	setView( VIEWS[viewIndex] );
}

/**
 * Setzt den Darstellungmodus 
 *
 * @param name Der Name der gewuenschten Darstellungart.
 * Dieser muss im Array VIEWS deklariert sein.
 *
 * @return void 
 */
public void setView( String name ) {

	int i = 0;
	while ( i < VIEWS.length )
	{
		if ( name.equals(VIEWS[i]) ) {
			viewIndex = i;
			repaint();
			return; 
		}

		i++;
	}

	// Wir benutzen eigentlich nur gueltige Namen, also...
	assert (i==0) : "Dude! What kind of name is '"+name+"' anyway?";

}

/**
 * getMarkupColor gibt die Farbe der aktuellen Operation des Models zurueck
 *
 * @return Color Die Zeichenfarbe der aktuellen Operation des Modells
 */
private Color getMarkupColor() {
	switch ( model.currOp[0] ) {
		case SortModel.NOOP:  return USUALCOLOR;
		case SortModel.READ:  return Color.GREEN;
		case SortModel.WRITE: return Color.RED;
		case SortModel.COMP:  return Color.CYAN;
		case SortModel.SWAP:  return Color.MAGENTA;
		default: return USUALCOLOR;
		}
}

// Accessor Methods
// Operations
/**
 * paintComponent zeichnet die Array-Werte in der Anzahl entsprechender 
 * Groesse ins Applet
 * @param g AWT-Graphics
 * @return void  
 */
public void paintComponent ( Graphics g) {

	int len, thick, tall;

	// Hoehe und Breite holen
	int win_w = getSize().width;
	int win_h = getSize().height;
	int basex = 0;
	int basey = 0;
	//Farben setzen
	g.setColor( BACKGROUND );
	g.fillRect( 0, 0, win_w, win_h );

	// Lock auf Laenge des Arrays und Inhalt 
	try
	{
	model.lock.acquire(); 
		len = model.getLength(); // Anzahl der Elemente im Array

		int step_w = win_w / len; // Darstellungsbreite eines Element
		int step_h = win_h / len;  // delta-height per value-point

		int array_w = step_w*len;
		int array_h = step_h*len;
		int offset_w = (win_w - array_w) / 2;
		int offset_h = (win_h - array_h) / 2;

		// lokalisiere viewIndex, damit nicht waehrend des Zeichnens
		// ein anderer View auftritt, und der switch unten konstant
		// bleibt ueber die Schleife
		int currView = viewIndex;

		Color markupColor = getMarkupColor();
	
		// Array aus Rechtecken konstanter Breite malen
		for (int i=0; i<len; i++) {

			int stepCount_h = model.fget(i);
			
			if ((i==model.currOp[1]) ||
			    (i==model.currOp[2]))
				   g.setColor( markupColor );
			else
				g.setColor( USUALCOLOR );

			// fill3DRect(x,y,width,height)
			// Ursprung = oben,links 
			switch (currView) {
			case 0: // Spots
				g.fill3DRect( 
					offset_w + i*step_w, 
					offset_h + (len-stepCount_h)*step_h,
					step_w, 
					step_h, 
					true );
				break;

			case 1: // Bars
				g.fill3DRect( 
					offset_w + i*step_w, 
					offset_h + (len-stepCount_h)*step_h,
					step_w, 
					stepCount_h*step_h, 
					true );
				break;
			}


		}
	
	} 
	catch (InterruptedException e) { /* ignore */ }
	finally {
		model.lock.release();
	}

	
}

/**
 * Startet das Neuzeichnen falls ein update anliegt.
 * Diese Methode wird ueberschrieben, um ggf. Scrollbalken
 * erscheinen zu lassen, wenn das Fenster nicht (mehr) gross
 * genug fuer das Array ist, weil das eine verkleinert oder
 * das andere vergroessert wurde.
 *
 * @param o Vom Interface benoetigt, aber hier nicht benutzt
 * @param arg Vom Interface benoetigt, aber hier nicht benutzt 
 * @return void  
 */
 public void update( Observable o, Object arg ) {

	// Scrollbars aktualisieren
	int len = model.getLength();
	setPreferredSize( new Dimension( len, len ) );
	revalidate();

	algoLabel.setText( model.getAlgoName() );

	repaint();

}

} // end of class ArrayView



