package jsortdemo.core;

/************************************************************************
  			SortModel.java - Copyright 
**************************************************************************/

import java.util.Observable;
import java.util.Random;
import jsortdemo.concurrent.*;


/**
 * Class SortModel.
 * <p>Project-Title: SortDemo --> SortModel</p>
 *
 * <p>Description: SortModel ist die Klasse, in der das zu sortierende Array und alles
 * was fuer die aktuell gewaehlte Konfiguration wichtig ist, erstellt und verwaltet
 * wird. Da andere Klassen auf Aenderungen des Modells reagieren koennen muessen,
 * erweitert SortModel die Java-Klasse Observable </p>
 *
 *
 * <p>Copyright: Copyright (c) 2004</p>
 *
 * <p>Company: </p>
 *

 * @author Mathis Ahrens, Yvonne von Bischopinck
 * @version 1.1
 */
public class SortModel extends java.util.Observable {

//Array zum Speichern der zu sortierenden Werte
private int[] array;
// Tmp-Array, um das etzte gewaehlte Wertearray zwischenzuspeichern
private int[] previousRandomArray;

//Der Algo nach dem das ganze sortiert werden soll
private SortAlgo algo;

public static final int NOOP  = -1;
public static final int READ  = 0;
public static final int WRITE = 1;
public static final int COMP  = 2;
public static final int SWAP  = 3;
public static final int MAXOP = 4;

public volatile int delay;
public volatile boolean paused;
public volatile boolean finished;

public volatile int[] opCount = new int[MAXOP];
public volatile int[] currOp = new int[3];

final public ReentrantLock lock;
final private CondVar condActivated;


// Constructors
/**
 * Konstruktor. Versucht den Algo zu laden und setzt die Groesse des Arrays fest 
 * Aenderungen reagieren zu koennen
 * @param algoName Der Name des zu verwendenden Algos
 * @param size Die Groesse des zu sortierendem Array
 * @param delay Die Verzoegerung pro Schritt in Millisekunden
 */
public SortModel( String algoName, int size, int delay ) {

	// versuche den angegebenen Algorithmus zu laden
	if ( ! setAlgo(algoName) )
	{
		System.out.println("No Sorting-Algo named '"+
				algoName+"' could be loaded.\n");

		System.exit(0);
	}

	resize(size);
	this.delay = delay;

	currOp[0] = NOOP;
	currOp[1] = -1;
	currOp[2] = -1;

	lock = new ReentrantLock();
	condActivated = new CondVar( lock );
	paused = true;

}


/**
*
* Liefert den Namen des aktuellen Algorithmus.
* Das ist der String, der setAlgo() zum Bauen
* der aktuellen Algo-Klasse uebergeben wurde.
*
* @return String
*/
public String getAlgoName ( ) {

	assert algo!=null : "Algo is null";

	return algo.getName();

}
/**
*
* Versucht, den angegebenen Algorithmus zu Instanziieren.
* Der Parameter wird an Leerzeichen in Woerter aufgeteilt,
* und das erste Wort als Klassenname fuer den zu ladenden
* Algorithmus verwendet.
* Danach werden alle Woerter der Instanz mittels initArgs()
* zwecks eventueller Parsung uebergeben.
* Anschliessend wird ein ggf. laufender Algo gestoppt und
* durch die neue Instanz ersetzt.
*
* @param name Zeichenkette mit dem Namen des zu setzenden Algorithmus
* @return boolean 'True' bei Erfolg, sonst 'false'
*/
public boolean setAlgo ( String name ) {

	try {

		// Leerzeichen als Wort-Separator
		String[] algoArgs = name.split(" ");
		// erstes Wort als Klassenname
		String fullName = "jsortdemo.algos."+algoArgs[0];

		System.out.println("Loading Algo:"+fullName);
		SortAlgo newAlgo = (SortAlgo)
			Class.forName(fullName).newInstance();

		newAlgo.setName( name );
		//newAlgo.initArgs( algoArgs );


		// Ist der Algortihmus am Sortieren oder zumindest
		// nur pausiert, so wird er per interrupt beendet.
		if ( isAlive () ) {
			algo.interrupt();
			algo.join();
			algo = null;
		}

		// neuen Algorithmus uebernehmen
		algo = newAlgo;
		algo.setParent(this);
		finished = false;

		return true;

	} catch (Exception e) {

		//TODO
		System.out.println("Oops, Exception in setAlgo() caused by: " + name );
		e.printStackTrace();
		return false;
	}
}

/**
* Anzahl der Elemente im Array
* @return int
*/
public int getLength() {
	return array.length;
}

/**
*
* Erzeugt ein neues leeres Array der angegebenen
* Laenge.
*
* Verlaengern oder Verkuerzen des Arrays macht
* keinen Sinn, da es nicht sinnvoll gefuellt waere.
*
* In dem Sonderfall, dass die neue Laenge genau
* der alten entspricht, bleibt alles beim Alten.
*
* @param newSize Die gewuenschte Groesse des Arrays
* @return void
*/
public void resize( int newSize ) {

	if ( (array==null) || ( newSize != getLength() ))
	{
		array = new int[newSize];
		previousRandomArray = new int[newSize];
		setArrayToRandom();
	}
}

/**
* 
* laedt ein sortiertes Feld 
* 
*/
public void setArrayToSorted () {

	int len = getLength();

	for (int i=0; i<len; i++)
	{
		array[i] = 1+i; 
	}
	requestRepaint();
}

/**
* 
* laedt ein verkehrt sortiertes Feld 
* 
*/
public void setArrayToReverseSorted () {

	int len = getLength();

	for (int i=0; i<len; i++)
	{
		array[i] = len - i; 
	}
	requestRepaint();
}

/**
* 
* laedt die letzte Zufallssortierung erneut
* (beim ersten Mal eine im Konstruktor erzeugte)
* 
*/
public void setArrayToPreviousRandom () {

	for (int i=0; i<getLength(); i++)
	{
		array[i] = previousRandomArray[i];
	}
	requestRepaint();
}

/**
* 
* laedt die letzte Zufallssortierung erneut
* (beim ersten Mal eine im Konstruktor erzeugte)
* 
*/
public void setArrayToRandom () {

	setArrayToReverseSorted();

	Random rgen = new Random();  // Random number generator
	int len = getLength();

	// Zufallsfeld durch beliebiges Vertauschen
	for (int i=0; i<len; i++) {
		int randomPosition = rgen.nextInt(len);
		int temp = array[i];
		array[i] = array[randomPosition];
		array[randomPosition] = temp;
	}

	// Feld fuer spaetere Resets speichern
	for (int i=0; i<getLength(); i++)
		previousRandomArray[i] = array[i];

	requestRepaint();
}


/**
* 
* Algo ist nicht NULL und noch nicht
* beendet.
*
* @return boolean  
*/
public boolean isAlive() {

	if ( algo == null ) 
		return false;
	else 
		return algo.isAlive();

}


/**
* 
* Algo ist nicht NULL, noch nicht
* beendet, aber gerade pausiert
*
* @return boolean  
*/
public boolean isPaused() {

	return ( isAlive() && paused ); 

}

/**
* 
* Algo ist nicht NULL, noch nicht
* beendet und nicht pausiert
*
* @return boolean  
*/
public boolean isSorting() {

	return ( isAlive() && ! paused ); 

}


/**
* start()et einen neuen Algorithmus.
*
* @return void
*/
public void startSortAlgo() {

	assert ( algo != null ) : "Assertion failed: algo is NULL!";

	// Thread nur starten, wenn er nicht gerade laeuft
	if ( ! isAlive() )
	{
		// Einen bereits durchgelaufenen Thread neu initiieren
		if ( finished )
			resetSortAlgo();

		System.out.println ("Starting...");

		paused = false;
		currOp[0] = NOOP;
		currOp[1] = -1;
		currOp[2] = -1;


		algo.start();

	}
}

/**
* Setzt das Pause-Flag (wenn der neue Wert ungleich dem aktuellen ist).
* Beim Wechsel von paused zu !paused wird der Sortier-Thread davon
* benachrichtigt.
*
* @return void
*/
public void setPaused( boolean shouldPause )
{
	if (( ! isAlive() ) ||
	( shouldPause == paused )) return;

	paused = shouldPause;

	// nach einer Pause den Algo-Thread aufwecken
	if ( ! shouldPause )
		condActivated.signal();

}

/**
* setzt die aktuelle Operation auf 0, weil der Algo mit dem Sortieren fertig ist
* und informiert die BEobachter von der Aenderung
* @return void
*/
public void algoFinished() {
	System.out.println("Algo finished sorting.");
	finished = true;
	currOp[0] = NOOP;
	requestRepaint();
}

/**
*
* Der aktuelle Sortieralgorithmus wird durch einen
* Aufruf von setAlgo( aktuellerAlgo ) getoetet und
* durch eine neue Instanz seiner Klasse ersetzt.
* Die Zaehler werden zurueckgesetzt, aber es wird
* kein neues Feld geladen.
*
* @return void
*/
public void resetSortAlgo() {

	if ( setAlgo( getAlgoName() ) ) {
		for (int i=0; i<MAXOP; i++)
			opCount[i] = 0;
	}

}


/**
* Gibt den Wert an einem bestimmten Index des Arrays zurueck
*
* @param index Der Index des gewuenschten Wertes
* @return int Der angeforderte Wert
*/
public int fget( int index ) {
	return array[index];
}


/**
* Testet einen Index darauf, ob er innerhalb der
* Feldgrenzen liegt (mittels assertion).
*
* Benoetigte Compiler-Flags:
* 	-source 1.4 (keyword 'assert')
* 	-ea (= enable assertions)
*
* @param int Index Der zu testende Index
* @return void
*/
private void assertRange( int index ) {
	assert (index>=0) :
		"Assertion failed: array index("+index+") is < 0";
	assert (index<array.length) :
		"Assertion failed: array index("+index+") is >= length";
}


/**
* Informiert alle Beobachter, falls ein Repaint erforderlich ist
* @return void
*/
private void requestRepaint() {

	// Beobachtern ein Refresh empfehlen
	setChanged();
	notifyObservers();


}


/**
* Zugriffsfunktion (fuer Sortieralgorithmen) zum
* Auslesen eines Array-Wertes.
* Diese Funktion sollte _nur_ von Sortieralgorithmen
* aufgerufen werden, da sie den Lesezaehler
* inkrementiert und den ausfuehrenden Thread fuer
* delay Millisekunden schlafen legt.
* Anschliessend werden registrierte Beobachter benachrichtigt.
*
* @param index Der Index, dessen Wert geholt werden soll
* @return int Der Wert der am uebergebenen Index steht
 *@throws InterruptedException
*/
public int get( int index ) throws InterruptedException {

	int result;

	lock.acquire();
	try
	{

		while (paused)
			condActivated.await();

		assertRange( index );

		opCount[READ]++;

		result = array[index];

	} finally {
		lock.release();
	}

	requestRepaint();

	// Ergebnis erst nach delay Millisekunden zurueckgeben
	Thread.currentThread().sleep(delay);	

	return result;  
}


/**
 * Schreibt einen Wert an einen bestimmten Index im Array
 *
 *@param index Der Index, an dessen Stelle der Wert geschrieben werden soll
 *@param value Der Wert der an den Index geschrieben werden soll
 *@return void
 */ 
public void put( int index, int value ) throws InterruptedException {

	lock.acquire();
	try
	{

		while (paused) 
			condActivated.await(); 

		assertRange( index );

		opCount[WRITE]++;

		array[index] = value;

	} finally {
		lock.release();
	}

	requestRepaint();

	// Erst nach delay Millisekunden zurueckkehren
	Thread.currentThread().sleep(delay);	

}


} // end of class SortModel

